
package localstuff;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.ensembl.datamodel.ExternalRef;
import org.ensembl.datamodel.Gene;
import org.ensembl.datamodel.Location;
import org.ensembl.datamodel.ProteinFeature;
import org.ensembl.datamodel.Transcript;
import org.ensembl.datamodel.Translation;
import org.ensembl.driver.AdaptorException;
import org.ensembl.driver.CoreDriver;
import org.ensembl.driver.DriverManager;
import org.ensembl.driver.GeneAdaptor;
import org.ensembl.driver.SequenceAdaptor;

public class QueryWrapper {
	
	HashMap ipiIdMap        = new HashMap();
	StringBuffer buf        = null;
	StringBuffer fst        = null;
	StringBuffer coOrds     = null;
	CoreDriver driver           = null;
	GeneAdaptor geneAdaptor = null;
	SequenceAdaptor seqAd   = null;
	
	int hCount          = 0;
	
	public QueryWrapper(String configFilename) throws AdaptorException {
		CoreDriver driver = DriverManager.loadDriver(configFilename);
		geneAdaptor   = driver.getGeneAdaptor();   
		seqAd         = driver.getSequenceAdaptor();
		System.out.println("CoreDriver connection: " + driver.toString());
		System.out.println("CoreDriver configuration: " +driver.getConfiguration());
		System.out.println();
		buf           = new StringBuffer();
		fst           = new StringBuffer();
		coOrds        = new StringBuffer();
		coOrds.append("#\tipi\tens_gene_id\tensprot_id\tuniprot_id\tcds_start\tcds_stop\t3_p_bp_remain\tfeat_name\tfeat_pep_start\tfeat_pep_stop\tfeat_nt_start\tfeat_nt_stop\n");		
	}
		
	public void getGene(String id) throws AdaptorException {
		if(ipiIdMap.containsKey(id)){
			IpiEntity ipi = (IpiEntity)ipiIdMap.get(id);
			if(ipi.getEnsGeneId()==null || ipi.getEnsGeneId().length()==0){					
				return;
			}
		}
		else {
			return;
		}
		IpiEntity curr = (IpiEntity)ipiIdMap.get(id);
		
	   Gene gene = geneAdaptor.fetch(curr.getEnsGeneId());
	   
	   List transcriptsList = gene.getTranscripts();
	   Iterator transcripts = transcriptsList.iterator();
		while(transcripts.hasNext()){    
			Transcript trans = (Transcript)transcripts.next();
			Location loc = trans.getCDNALocation();	
			String CDNA = seqAd.fetch(loc).getSequence().getString();
			if (gene.getAccessionID().equals("ENSMUSG00000035305")) {
			  System.out.println(loc);
			  System.out.println(CDNA);
			}
			  
			int cdsLen = CDNA.length();
			Translation protein = trans.getTranslation();
			//StringBuffer cds = new StringBuffer();
			/**
			Iterator exons = trans.getExons().iterator();
			while(exons.hasNext()){
				Exon exon = (Exon)exons.next();
				exon.getSequence().getString();
				cds.append(exon.getSequence().getString()+" ");
			}**/				
			if(curr.getEnsProtId().equals(protein.getAccessionID())){
				if(trans.getThreePrimeUTR().size()>0 || trans.getFivePrimeUTR().size()>0){
					hCount++;		
				}
				int fiveLen = 0;
				if(trans.getFivePrimeUTR().size()>0){
					int ucount = trans.getFivePrimeUTR().size();
						for(int i=0;i<ucount;i++){
							Location fiveputr = (Location)trans.getFivePrimeUTR().get(i);
							fiveLen +=fiveputr.getLength();
							String fiveseq = seqAd.fetch(fiveputr).getSequence().getString();
						}				
				}
				int threeLen = 0;
				if(trans.getThreePrimeUTR().size()>0){
					int ucount = trans.getThreePrimeUTR().size();
						for(int i=0;i<ucount;i++){
							Location threeputr = (Location)trans.getThreePrimeUTR().get(i);
							threeLen+=threeputr.getLength();
							String fiveseq = seqAd.fetch(threeputr).getSequence().getString();
						}			
				}
				int cdsStart = fiveLen+1;
				int cdsStop  = cdsLen - threeLen+1;
				buf.append(cdsLen + " " + cdsStart + " " + cdsStop);
				buf.append("\nCDS ");
				String seq = CDNA.substring(cdsStart-1,cdsStop-1);
				
				buf.append( 
					seq.substring(0,3).toUpperCase() + " " + 
					seq.substring(seq.length()-3).toUpperCase() + " " + 
					seq.substring(0,10).toLowerCase() + " " + 
					seq.substring(seq.length()-10).toLowerCase() + "    "+
					trans.getAccessionID()+ " "+
					protein.getAccessionID()+" "+
					curr.getEnsGeneId() +  " " +
					curr.getEnsProtId()+ " " + 
					curr.getIpiId()+" " + 
					curr.getUniprotId() + "\n"
					);
				Iterator refs = 	trans.getExternalRefs().iterator();
				while(refs.hasNext()){
					ExternalRef ref = (ExternalRef)refs.next();
					if(ref.getExternalDatabase().getName().equals("GO")){
						buf.append("ref: "+ref.getDisplayID()+"\n");
					}
				}
				fst.append("loc: "+trans.getLocation().toString()+"\n");
				fst.append(">" + curr.getIpiId() + " " + curr.getEnsGeneId() + " "+curr.getEnsProtId()+ " "+ curr.getUniprotId() + " "+cdsStart + " "+cdsStop + " "+ (cdsLen-cdsStop+1)+"\n");
				fst.append(CDNA+"\n");
				
				buf.append(">"+curr.getEnsProtId()+"\n"+protein.getPeptide()+"\n");
				//coOrds.append(">" + curr.getIpi() + " " + curr.getEnsG() + " "+curr.getEnsP()+ " "+ curr.getUnip() + " "+cdsStart + " "+cdsStop + " "+ (cdsLen-cdsStop+1)+"\n");
				String pre = curr.getIpiId() + "	" + curr.getEnsGeneId() + "	"+curr.getEnsProtId()+ "	"+ curr.getUniprotId() + "	"+cdsStart + "	"+cdsStop + "	"+ (cdsLen-cdsStop+1) + "	";					
				Iterator doms = protein.getProteinFeatures().iterator();
					while(doms.hasNext()){
						ProteinFeature feat = (ProteinFeature)doms.next(); 
						int fLen     = (feat.getPeptideEnd() - feat.getPeptideStart()) * 3;
						int startPos = cdsStart + ((feat.getPeptideStart()-1)*3);
						coOrds.append( pre +"	"+
						feat.getDisplayName()+"	" + 
						feat.getPeptideStart()+"	"+
						feat.getPeptideEnd()+"	"+
						startPos + "	"  + (startPos + fLen) + "\n"
						);
					}
										
			}
		}
	}
		
	
}
