/*
 * Copyright (C) 2001 EBI, GRL
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package localstuff;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.ensembl.compara.datamodel.GenomeDB;
import org.ensembl.compara.driver.ComparaDriver;
import org.ensembl.datamodel.CoordinateSystem;
import org.ensembl.datamodel.Exon;
import org.ensembl.datamodel.FeaturePair;
import org.ensembl.datamodel.Gene;
import org.ensembl.datamodel.KaryotypeBand;
import org.ensembl.datamodel.Location;
import org.ensembl.datamodel.Sequence;
import org.ensembl.datamodel.Translation;
import org.ensembl.driver.AdaptorException;
import org.ensembl.driver.CoreDriver;
import org.ensembl.driver.CoreDriverFactory;
import org.ensembl.driver.ExonAdaptor;
import org.ensembl.driver.FeatureIterator;
import org.ensembl.driver.LocationConverter;
import org.ensembl.driver.LocationFeatureIterator;
import org.ensembl.driver.TranslationAdaptor;
import org.ensembl.registry.Registry;
import org.ensembl.util.StringUtil;
import org.ensembl.util.SystemUtil;

public class Tmp { 

  private static void dump(String prefix,  Iterator iter) {
    while(iter.hasNext()) {
      //iter.next();
      System.out.println(prefix +"\n"+ iter.next());
    }
  }

  public static void main(String[] args) throws Exception {
    

    
  }

  private static CoreDriver getCoreDriver(String group) throws AdaptorException {
    Registry r = Registry.createDefaultRegistry();
    return r.getGroup(group).getCoreDriver();
 
  }

  private static ClassLoader jarClassLoader(String version ) throws Exception {
    String jarPath = "file:///home/craig/java/lib_archive/ensj-"+version+".jar";
    URL jarURL = new URL(jarPath);
    if (!new File(jarURL.getFile()).exists())
      throw new FileNotFoundException(jarPath);
    return new URLClassLoader(new URL[]{jarURL},null);
  }
  
  
  
  
  
  public static void mouseGeneOrthologueProblem() throws Exception {
    
    ComparaDriver comparaDriver = Registry.createDefaultRegistry().getGroup("compara").getComparaDriver();
    List gdbs = comparaDriver.getGenomeDBAdaptor().fetch();

    for (int i = 0; i < gdbs.size(); i++) {

      GenomeDB gdb =(GenomeDB) gdbs.get(i);
      String targetSpecies = (String) gdb.getName();

      List hits = comparaDriver.getMemberAdaptor().fetch("Mus musculus",
                       //new String[]{"ENSG00000018907"}, targetSpecies);
          new String[]{"ENSMUSG00000018907"}, targetSpecies);

      System.out.print(targetSpecies);

      for (int j = 0; j < hits.size(); j++) {

         FeaturePair p = (FeaturePair) hits.get(j);

        System.out.print("\t" +p.getDisplayName()+"  "+p.getHitLocation() );

       }
     System.out.println();

    }

  }
  
  public static void peptideBug() throws Exception {
    System.out.println(SystemUtil.environmentDump());
    CoreDriver d = getCoreDriver("pan_troglodytes");
    Translation tr1 = d.getTranslationAdaptor().fetch("ENSPTRP00000008683");
    System.out.println(tr1);
    System.out.println(tr1.getSequence());           
    System.out.println(tr1.getPeptide());
  }
  
  public static void main7(String[] args) throws Exception {
    
    // Get a driver for the latest human db on ensembldb.ensembl.org
    Registry r = Registry.createDefaultRegistry();
    CoreDriver d = r.getGroup("human").getCoreDriver();
    
    // iterate over all the genes
    Iterator gi = d.getGeneAdaptor().fetchIterator();
    while(gi.hasNext()) {
      
      Gene g = (Gene) gi.next();
      
      // skip unkown genes
      if (!g.isKnown()) continue;
      
      // get the gene location, this is from the start of it's first exon (including potential UTR)
      // to the end of the last exon (again including potential UTR)
      Location l = g.getLocation();
      
      
      // add 200 bases upstream
      l = l.transform(-200, 0);
      
      // fetch the sequence for the specified location
      Sequence s = d.getSequenceAdaptor().fetch(l);
      System.out.println(g.getAccessionID() + "\t" + g.getInternalID() + "\t" + l + "\t" + s.getString());
      
      //break;
    }
    
  }
  
  public static void main6(String args[]) throws Exception {
    
    System.out.println("bob");
    
    Registry r = Registry.createDefaultRegistry();
    Gene g = r.getGroup("dog").getCoreDriver().getGeneAdaptor().fetch("ENSCAFG00000005212");
    System.out.println(g);
  }
  
  public static void main5(String args[]) throws Exception {
    Registry r = Registry.createDefaultRegistry();
    //CoreDriver d = r.getGroup("human").getCoreDriver();
    CoreDriver d = CoreDriverFactory.createCoreDriver("/home/craig/.ensembl/ensembldb_human.properties");
    System.out.println("Ngenes: " + d.getGeneAdaptor().fetch(new Location("chromosome:Y")).size());
//    System.out.println(d);
//    fetchBands(d, new CoordinateSystem("chromosome"), "Y");
//    System.out.println("done");
  }
  
  public static HashMap fetchBands(CoreDriver coreDriver, CoordinateSystem coordinateSystem,String chromosomeName) throws AdaptorException
  {
    // missing ones are: p11.32, p11.31 and q.12.
    
              String allBandInfo = null;
              HashMap bandMap = new HashMap();
              List l = coreDriver.getKaryotypeBandAdaptor().fetch(coordinateSystem,chromosomeName);
      System.out.println("Chromosome " + chromosomeName + " has " + l.size()+ " bands.");

              for (int i = 0; i<l.size(); ++i)
              {
     KaryotypeBand kb = (KaryotypeBand) l.get(i);
     String name = kb.getBand();
     Location loc = kb.getLocation();
     int start = loc.getStart();
     int end = loc.getEnd();
     allBandInfo = name+"#"+start+"#"+end;
     System.out.println(chromosomeName+ '\u0009' +name+'\u0009' + start + '\u0009' + end);
     bandMap.put(name,allBandInfo);
              }
              return bandMap;
  }
  
  public static void main3(String args[]) throws Exception {
  
  
  
  int[] ids = new int[]{38138, 
                         38139, 
                         38140, 
                         38141, 
                         38142, 
                         38143, 
                         38144, 
                         38145, 
                         38146, 
                         38147, 
                         38148, 
                         38149, 
                         38150, 
                         38151, 
                         38152, 
                         38153, 
                         38154, 
                         38155, 
                         38156};
  
  //ids= new int[]{38148};
  
//  String p = "ENSMUSP00000081023";
//    //"ENSMUSP00000081013";
  Registry r = Registry.createDefaultRegistry();
  TranslationAdaptor ta = r.getGroup("mouse").getCoreDriver().getTranslationAdaptor();
  
//  Translation tn = ta.fetch("ENSMUSP00000081004");
//  String seq = tn.getSequence().getString();
//  String pep = SequenceUtil.dna2protein(seq,true);
//  String pepLong = SequenceUtil.dna2protein(seq,false);
//  List atts = tn.getAttributes();
//  System.out.println(atts.size());
//  for (int i = 0; i < atts.size(); i++) {
//    System.out.println(atts);
//  }
//  System.out.println("seq " + seq.length() + "\t" + seq);
//  System.out.println("pep " + pep.length() + "\t" + pep);
//  System.out.println("pepLong " + pepLong.length() + "\t" + pepLong);
//  int count = 0;
//  for (int i = 0; i < pepLong.length(); i++) {
//    if (pepLong.charAt(i)=='*'){
//      System.out.println(i);
//      count++;
//    }
//  }
//  System.out.println("* count " + count);
  
  for (int i = 0; i < ids.length; i++) {
    Translation tn = ta.fetch(ids[i]);
    //System.out.println(tn.getAttributes());
    //System.out.println(tn.getSequence());
    //System.out.println("len=" + tn.getSequence().getString().length()/3);  
    try {
      System.out.println(tn.getAccessionID()+"\t"+tn.getPeptide());
    } catch (RuntimeException e) {
      //System.out.println(e.getMessage());
      // TODO Auto-generated catch block
      System.err.println(ids[i]);
      System.err.println(tn.getAccessionID());
      e.printStackTrace();
    }  
  }
  
  
//  Registry r = new Registry(new RegistryLoaderIni(new String[]{"/tmp/ensembldb.ini"}, new ServerDriverFactory(), true));
//  System.out.println(r);
//  r.getGroup("human").getCoreDriver().getGeneAdaptor().fetchIterator().next();
  
  
//  Class.forName("com.mysql.jdbc.Driver");
//  //nnection c = DriverManager.getConnection("jdbc:mysql://127.0.0.1:23365/vega_mart_33", "ensro","");
//  Connection c = DriverManager.getConnection("jdbc:mysql://ensembldb.ensembl.org/vega_mart_34?user=anonymous");
//  System.out.println(c.getMetaData());
  
//  
//  	Location l = new Location("chromosome:1:100-200");
//  	Location l2 = l.transform(-10,10);
//  	System.out.println(l.toString()+"\t"+l2);
//  	Registry r = Registry.createDefaultUserRegistry();
//  	System.out.println(Registry.getDefaultUserRegistryURL());
  	//System.out.println(r.getGroups());
  	//Registry.isDefaultUserRegistryAvailable();
  
//  Registry r = Registry.createDefaultRegistry();
//  CoreDriver d = r.getGroup("human").getCoreDriver();
//  
//  List bands = d.getKaryotypeBandAdaptor().fetch(new CoordinateSystem("chromosome"), "19");
//  System.out.println(StringUtil.toString(bands,"\n"));
//  System.out.println(d.getDatasource());
  
  
//  CoreDriver steve = Registry.createDefaultUserRegistry().getGroup("steve").getCoreDriver();
//  List transcripts = steve.getPredictionTranscriptAdaptor().fetch(new Location("chromosome:19:1-200000"));
//  for (int i = 0; i < transcripts.size(); i++) {
//    PredictionTranscript t = (PredictionTranscript) transcripts.get(i);
//    System.out.println(t.getDisplayName() +"\t"+t.getExons().iterator().next());
//  }
  
//  LruCache cache = new LruCache(10);
//  
//  cache.put(new Long(1), 1l);
//  cache.get(1l);
//  System.out.println(cache.listSize() +"\t"+cache);
//  
//  cache.put(new Long(2), 2l);
//  cache.get(1l);
//  System.out.println(cache.listSize() +"\t"+cache);
//  cache.get(2l);
//  System.out.println(cache.listSize() +"\t"+cache);
  
//  cache.put(new Long(3), 3l);
////  cache.get(1l);
////  cache.get(2l);
////  cache.get(3l);
//  System.out.println(cache.listSize() +"\t"+cache);
    
//    ConnectionPoolDataSource ds 
//    	= new ConnectionPoolDataSource("com.mysql.jdbc.Driver"
//    	    ,"jdbc:mysql://ensembldb.ensembl.org:3306/homo_sapiens_core_32_35e?autoReconnect=true&zeroDateTimeBehavior=round"
//    	    ,"anonymous"
//    	    ,null
//    	    ,3);
//    
//    Connection conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();
//    
//    conn = ds.getConnection("homo_sapiens_variation_32_35e");
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();
//    
//    conn = ds.getConnection("homo_sapiens_variation_32_35e");
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();
//
//    conn = ds.getConnection("ensembl_compara_32");
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();    
//
//    conn = ds.getConnection("ensembl_compara_32");
//    conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();  
//    
//    conn = ds.createDataSourceProxy("ensembl_compara_32").getConnection();
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();       
//    
//    conn = ds.createDataSourceProxy("ensembl_compara_32").getConnection();
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\t"+conn+"\t"+ds);
//    conn.close();       
//    
//   
//    conn = ds.createDataSourceProxy("ensembl_compara_32").getConnection();
//    //conn = ds.getConnection();
//    System.out.println(conn.getCatalog()+"\tzzz"+conn+"\t"+ds);
//    conn.close();     
//    Registry registry = Registry.createDefaultRegistry();
//    
//
//// get a driver for the latest human core database
// CoreDriver coreDriver = registry.getGroup("human").getCoreDriver();
// System.out.println(coreDriver);
    
    //Registry registry = Registry.createDefaultRegistry();
    
//    System.out.println(Registry.getDefaultRegistryURL());
//    System.out.println(Registry.getDefaultUserRegistryURL());
    
    
//    DriverGroup dg = registry.getGroup("compara");
//		ComparaDriver compara = dg.getComparaDriver();
//
// 		List gdbs = compara.getGenomeDBAdaptor().fetch();
// 		System.out.println(StringUtil.toString(gdbs));

    
    //CoreDriver cd =
    // Registry.createDefaultUserRegistry().getGroup("human").getCoreDriver();
//    CoreDriver cd =
// CoreDriverFactory.createCoreDriver("ensembldb.ensembl.org",3306,
// "homo_sapiens_core_31_35d", "anonymous", null);
//    
//    List genes = cd.getGeneAdaptor().fetch(new Location("chromosome:20:20m-21m"),
// true);
//    ((Gene)genes.get(0)).getTranscripts().get(0);
//    
//    Gene g = cd.getGeneAdaptor().fetch(92004);
//    g.getTranscripts().get(0);
    
    //System.out.println(cd);
    
    //Location loc = new Location("chromosome_NCBI35:1:1-1000:1");
    //System.out.println(loc.toString(true,true));
    
//    List rs = cd.getRepeatFeatureAdaptor().fetch( loc);
//    for (int i = 0; i < rs.size(); i++) {
//      RepeatFeature r = (RepeatFeature) rs.get(i);
//      System.out.println(r);
//    }
    
//    RepeatFeature rf = cd.getRepeatFeatureAdaptor().fetch(6173413);
//    Location loc = rf.getLocation();
//    System.out.println(loc.toString(true,true));
//    loc = cd.getLocationConverter().convert(rf.getLocation(),"chromosome",
// true,true, false);
//    System.out.println(loc.toString(true,true));
//    boolean hasGap=false;
//    for(; !hasGap && loc!=null; loc=loc.next())
//      hasGap = loc.isGap();
//    if (hasGap)
//      System.out.println("Only part of repeat on chromosome");
//    else
//      System.out.println("Do something when repeat completely present on
// chromsome");
//    
//    
//    System.out.println(loc.toString(true,true));
//    
//    
//    Location loc2 = cd.getLocationConverter().convert(loc, "contig");
//    System.out.println(loc2.toString(true,true));
//    
//    Location loc3 = cd.getLocationConverter().convert(loc2, "chromosome",
// true,true, false).mergeAdjacentNodes();
//    System.out.println(loc3.toString(true,true));
    
    
//    ZipFile zf = new
// ZipFile("/home/craig/dev/ensembl/modules/t/test_genome.zip");
//    Enumeration enum = zf.entries();
//    while (enum.hasMoreElements()) {
//      ZipEntry e = (ZipEntry) enum.nextElement();
//      //System.out.println(e);
//      System.out.println(e.getName());
//    }
    
//    System.out.println(Tmp.class.getClassLoader().getResource("org/ensembl/Example.class"));
    
//    ComparaDriver compara =
// Registry.createDefaultUserRegistry().getGroup("compara").getComparaDriver();
//    
//    List gds = compara.getGenomeDBAdaptor().fetch();
//    for (int i = 0; i < gds.size(); i++) {
//      GenomeDB gdb =(GenomeDB) gds.get(i);
//      String targetSpecies = (String) gdb.getName();
//      // List hits = compara.getMemberAdaptor().fetch("Homo sapiens", new
// String[]{"ENSG00000118961"}, "Gallus gallus");
//      List hits = compara.getMemberAdaptor().fetch("Homo sapiens", new
// String[]{"ENSG00000118961"}, targetSpecies);
//      System.out.print(targetSpecies);
//      for (int j = 0; j < hits.size(); j++) {
//        FeaturePair p = (FeaturePair) hits.get(j);
//        System.out.print("\t" +p.getHitDisplayName());
//      }
//      System.out.println();
//    }
    
    
//    List gdbs = compara.getGenomeDBAdaptor().fetch();
//    for (int i = 0; i < gdbs.size(); i++) {
//      Object o = (Object) gdbs.get(i);
//      System.out.println(o);
//    }
    
//    CoreDriver core_driver =
//    CoreDriverFactory.createCoreDriverUsingDatabasePrefix("ensembldb.ensembl.org",
// 3306, "homo_sapiens_core", "anonymous", null);
//    List genes = core_driver.getGeneAdaptor().fetch(new
//    Location("chromosome:2:20m-21m:1"));
//    System.out.println(genes.size());
//    for (int i = 0; i < genes.size(); i++) {
//      Gene g = (Gene) genes.get(i);
//      System.out.println(g.getAccessionID());
//    }
    
    
//    Class.forName("com.mysql.jdbc.Driver").newInstance();
//    Connection conn =
// java.sql.DriverManager.getConnection("jdbc:mysql://ensembldb.ensembl.org/",
// "anonymous",null);
//    for(ResultSet rs = conn.getMetaData().getCatalogs(); rs.next();)
//      System.out.println(rs.getString(1));
    
//    System.out.println(Registry.isDefaultUserRegistryAvailable());
//    System.out.println(Registry.getDefaultUserRegistryURL());
//    Registry r = Registry.createDefaultUserRegistry();
//    List genes = r.getGroup("human").getCoreDriver().getGeneAdaptor().fetch(new
// Location("chromosome:DR52"), false);
//    IDSet ids = new IDSet(genes);
//    System.out.println(ids.size() + "\t" + ids);
    
//    Iterator iter =
// r.getGroup("human").getCoreDriver().getGeneAdaptor().fetchIterator(new
// Location("chromosome:Y"), false);
//    IDSet ids = new IDSet(iter);
//    System.out.println(ids.size() + "\t" + ids);
    
    //Class.forName("");

//    url= "jdbc:mysql://ensembldb.ensembl.org/?autoReconnect=true"

//    System.out.println(PropertiesUtil.stringToProperties(
//        "host=ensembldb.ensembl.org:user=anonymous:database_prefix=homo_sapiens_core",
// ":"));
    
//    CoreDriver cd =
// ComparaDriverFactory.createCoreDriver("resources/data/unit_test_rat.properties");
//    CoordinateSystem[] path = cd.getCoordinateSystemAdaptor().getMappingPath(new
// CoordinateSystem("chromosome"), new CoordinateSystem("contig"));
//    System.out.println(StringUtil.toString(path));
//    System.out.println(cd);
//    
//    int n = 0;
//    for (Iterator i=cd.getGeneAdaptor().fetchIterator(new
// Location("chromosome:10")); i.hasNext();) {
//      Gene g = (Gene) i.next();
//      n+=1;
//    }
//    
//    System.out.println(n);
  
//    Location l = new Location("chromosome:x:24573175-24595381");
//    Location l2 = cd.getLocationConverter().convert(l, new
// CoordinateSystem("contig"));
//    System.out.println(l);
//    System.out.println(l2);
//    
//    String s = cd.getSequenceAdaptor().fetch(l).getString();
//    System.out.println(Integer.toString(l.getLength())+" Expected \n" +
// s.length() + " got.");
//    System.out.println(s);

  }  public static void main2(String[] args) throws Exception {

    // CoreDriver for accessing the core database
    //    CoreDriver cd =
    // ComparaDriverFactory.createCoreDriver("resources/data/unit_test_core.properties");

    //    // create a driver for a variation variation
    //    VariationDriver vd =
    // VariationDriverFactory.createVariationDriver("resources/data/unit_test_variation.properties");
    //    // the variation driver needs a sister driver to access the core database
    //    vd.setCoreDriver(cd);
    //
    //    // Fetch the variation features from chromosome 20 in the range 20m to
    // 20.1m
    //    
    //    List vfs = vd.getVariationFeatureAdaptor().fetch(loc);
    //    for (int i = 0; i < vfs.size(); i++) {
    //      VariationFeature vf = (VariationFeature) vfs.get(i);
    //      System.out.println(loc + ":\t" + vf);
    //    }

    //    System.out.println(cd.getMiscFeatureAdaptor().fetch(1));// 3 atts
    //    System.out.println(cd.getMiscFeatureAdaptor().fetch(100000000));// null

    //    System.out.println(StringUtil.toString(((GeneAdaptorImpl)cd.getGeneAdaptor()).tables()));
    //    
    //    Location loc = new Location("chromosome:20:20m-20.01m");
    //    // Fetch the variations that overlap with introns and exons of a gene
    //    List gs = cd.getGeneAdaptor().fetch(loc);
    //    for (int j = 0; j < gs.size(); j++) {
    //      Gene g = (Gene) gs.get(j);
    //
    //        System.out.println(g.getAccessionID());
    //    }

    // String s = "abc#d";
    // System.out.println(StringUtil.toString(s.split("[\\|#]")));

    // String geneID = "ENSG00000130985";
    // Properties configProps = new Properties();
    // configProps.setProperty("ensembl_driver",
    // "org.ensembl.driver.impl.CoreDriverImpl");//
    // "org.ensembl.driver.plugin.standard.MySQLDriver");
    // configProps.setProperty("host", "ensembldb.ensembl.org");
    // configProps.setProperty("user", "anonymous");
    // configProps.setProperty("database", "homo_sapiens_core_29_35b");
    // CoreDriver driver = DriverManager.load(configProps);
    // System.out.println("C");

    // SequenceAdaptor sa=driver.getSequenceAdaptor();
    // GeneAdaptor ga=driver.getGeneAdaptor();
    // Gene myGene=ga.fetch(geneID);
    // List L=myGene.getExons();
    // System.out.print("is the list empty?" +L.isEmpty()+ "; ");
    // Iterator I=L.iterator();
    // int i=0;
    // while(I.hasNext()){
    // Object E=I.next();
    // Exon Ex=(Exon)E;
    // Location lo=Ex.getLocation();
    // int start=lo.getStart();
    // int end =lo.getEnd();
    // System.out.println(i+":start="+start + ";end="+end);
    // i++;
    //      
    // }

    // Gene g = driver.getGeneAdaptor().fetch("ENSG00000130985");
    // List ts = g.getTranscripts();
    // // we know there are 2 transcripts...
    // Transcript t0 = (Transcript) ts.get(0);
    // Transcript t1 = (Transcript) ts.get(1);
    //    
    // List geneExons = g.getExons();
    // for (int i = 0; i < geneExons.size(); i++) {
    // Exon e = (Exon) geneExons.get(i);
    // String exonInT0 = t0.getExons().contains(e) ? t0.getAccessionID() : "";
    // String exonInT1 = t1.getExons().contains(e) ? t1.getAccessionID() : "";
    // System.out.println(e.getAccessionID() + ":\t"+ exonInT0 + "\t"+ exonInT1
    // );
    // }

    // CoreDriverImpl d = new CoreDriverImpl("ensembldb.ensembl.org",
    // "homo_sapiens_core_28_35a", "anonymous");
    // System.out.println(d.getConnection().getMetaData().getDriverName().toLowerCase().matches(".*mysql.*"));

    // System.out.println(d.getTranslationAdaptor().fetch(96356).getPeptide());

    // CoreDriverImpl d = new CoreDriverImpl("ensembldb.ensembl.org",
    // "mus_musculus_core_28_33d", "anonymous");
    // List ts =
    // d.getGeneAdaptor().fetch("ENSMUSG00000035305").getTranscripts();
    // for (int i = 0; i < ts.size(); i++) {
    // Transcript t = (Transcript) ts.get(i);
    // System.out.println(d.getSequenceAdaptor().fetch(t.getCDNALocation()).getString());
    // }

    //    
    // try{
    // LoggingManager.configure("/home/craig/.ensembl/logging.properties");
    // logger.warning("stuff");
    // throw new RuntimeException("ooops");
    // } catch(Exception e) {
    // logger.log(Level.SEVERE, "Program fucked", e);
    // }

    // CoreDriver d = (CoreDriver)
    // DriverManager.loadEnsemblDriver("/home/craig/.ensembl/unit_test.properties");
    // List l = d.getAffyArrayAdaptor().fetch();
    // System.out.println(l);
    // OligoArray aa = d.getAffyArrayAdaptor().fetch("HG-U133_Plus_2");
    // System.out.println(aa);

    // d.getMiscFeatureAdaptor().fetchAll();

    // CoreDriver driver =
    // DriverManager.load("/home/craig/.ensembl/ensembldb_human.properties");
    //    
    // IDSet six = new IDSet(driver.getGeneAdaptor().fetch(new
    // Location("chromosome:6"), false));
    // IDSet dr52 = new IDSet(driver.getGeneAdaptor().fetch(new
    // Location("chromosome:DR52"), false));
    // IDSet justDr52 = new IDSet(dr52);
    // justDr52.removeAll(six);
    // IDSet justSix = new IDSet(six);
    // justSix.removeAll(dr52);
    // System.out.println("dr52 only" + justDr52);
    // System.out.println("six only" + justSix);

    // CoordinateSystem cs = new CoordinateSystem("chromosome");
    // Location chromloc = new Location(cs, "1", 10000000, 10000010);
    // try{
    // Location targetLoc = driver.getLocationConverter().convert(chromloc, cs);
    // SequenceAdaptor sa = driver.getSequenceAdaptor();
    // org.ensembl.datamodel.Sequence sequence = sa.fetch(targetLoc);
    //
    // String seq_code = sequence.getString();
    //
    // System.out.println(seq_code);
    // }
    // catch(Exception e){
    // return;
    // }

    // logger.warning("test mesg");

    // VariationDriver vd = (VariationDriver)
    // DriverManager.loadEnsemblDriver("/home/craig/.ensembl/unit_test_variation.properties");
    //
    // Variation v = vd.getVariationAdaptor().fetch(10012468);
    // System.out.println(v);
    // System.out.println(v.getAlleles());
    // /String s =
    // "contig:D87018.1.1.38756:17000-38756->contig:D87016.2.1.37115:12878-37115->contig:D86999.1.1.40778:6578-10583";
    // s = "contig:D87018.1.1.38756:17000-38756";
    // s = "contig:D87016.2.1.37115:12878-37115";
    // s =
    // "contig:D87016.2.1.37115:12878-37115->contig:D87016.2.1.37115:12878-37115";
    // s = "contig:D86999.1.1.40778:6578-10583";
    // s=
    // "contig:D87018.1.1.38756:17000-38756->contig:D87016.2.1.37115:12878-37115->contig:D86999.1.1.40778:6578-10583";
    // String[] ls = {
    // "chromosome:22:21m-21.05m",
    // "contig:D87018.1.1.38756:17000-38756->contig:D87016.2.1.37115:12878-37115->contig:D86999.1.1.40778:6578-10583"
    // };
    // for (int j = 0; j < ls.length; j++) {
    //
    // List gs = d.getGeneAdaptor().fetch(new Location(ls[j]));
    // System.out.println("Foung n genes:" + gs.size());
    // for (int i = 0; i < gs.size(); i++) {
    // Gene g = (Gene) gs.get(i);
    // System.out.println(g);
    // System.out.println(g.getTranscripts().get(0));
    // System.out.println(g.getExons().get(0));
    //
    // }
    // }
    // String name = "232702_at";
    // List aps = d.getAffyProbeAdaptor().fetch(name);
    //    
    // for (int i = 0; i < aps.size(); i++) {
    // OligoProbe ap = (OligoProbe) aps.get(i);
    //      
    // for (Iterator iter = ap.getAffyArraysContainingThisProbe().iterator();
    // iter
    // .hasNext();) {
    // OligoArray aa = (OligoArray) iter.next();
    // System.out.println(name + "\t" + ap.getInternalID() + "\t"
    // +aa.getInternalID() + "\t" + ap.getNameInArray(aa));
    //
    // }
    // }
    // System.out.println(PropertiesUtil.createProperties("resources/data/unit_test.properties"));

    // VariationDriver vdriver = new
    // MySQLVariationDriver("ensembldb.ensembl.org","homo_sapiens_variation_27_35a","anonymous");
    // vdriver.setCoreDriver(new
    // CoreDriverImpl("ensembldb.ensembl.org","homo_sapiens_core_27_35a","anonymous"));
    //	  
    // List vfs = vdriver.getVariationFeatureAdaptor().fetch(new
    // Location("chromosome:1:1-.3m"));
    // System.out.println("n vfs = " + vfs.size());
    // for (Iterator iter = vfs.iterator(); iter.hasNext();) {
    // VariationFeature vf = (VariationFeature) iter.next();
    // System.out.println(vf.getVariation());
    // }

    // System.out.println("Started...");
    // CoreDriver d = DriverManager.load("resources/data/unit_test.properties");
    // CoreDriver d = new
    // CoreDriverImpl("ensembldb.ensembl.org","homo_sapiens_core_27_35a","anonymous");
    // System.out.println(d);
    //	  
    // Location l = new Location("chromosome:DR52");
    // System.out.println("l="+l);
    //
    // Location[] dl = d.getLocationConverter().dereference(l);
    // System.out.println("dl="+StringUtil.toString(dl));

    // System.out.println(new Location(new
    // Location("c:1:10-20:-1").toString()));
    // System.out.println(new Location(new
    // Location("c:1:10-20:-1->c:1:30-40").toString()));
    // System.out.println(new Location(new
    // Location("c:1:10-20:-1->c:1:30-40->c:2:5k-10m").toString()).toString(true,true,false));

    // CoreDriver d = new
    // CoreDriverImpl("ensembldb.ensembl.org","homo_sapiens_core_25_34e","anonymous");

    // System.out.println();
    // Location l =
    // d.getTranscriptAdaptor().fetch("ENST00000259271").getTranslation().getCodingLocation();
    // System.out.println(l);
    // System.out.println(l.relative(837).toString(true,true,true));
    // System.out.println(l.relative(840).toString(true,true,true));

    // SimpleTimer t = new SimpleTimer().start();
    // Class.forName("com.mysql.jdbc.Driver");
    // int n = 100;
    // for (int i = 0; i <n; i++) {
    // Connection conn = java.sql.DriverManager
    // .getConnection(
    // "jdbc:mysql://ensembldb.ensembl.org/homo_sapiens_core_26_35?autoreconnect=true",
    // "anonymous", null);
    // conn.createStatement().execute("Show databases");
    // conn.close();
    // }
    // long time = t.stop().getDuration();
    // System.out.println(""+n+"\t"+time/n+" (" + time +")");

    // LoggingUtil.setRootLoggerHandlersLevelsToINFO();
    //		
    // //Logger.getLogger("org.ensembl.util.ConnectionWrapper").setLevel(Level.FINE);
    // logger.setLevel(Level.FINE);
    // logger.fine("fuck");
    // logger.fine("it");
    //
    //  	
    //  	
    // CoreDriver d = DriverManager.load("resources/data/unit_test.properties");

    // System.out.println(d.getLocationConverter().fetchComplete(new
    // Location("chromosome:22")));
    // Thread.sleep(2000);
    // System.out.println(d.getLocationConverter().fetchComplete(new
    // Location("chromosome:22")));
    //				
    // Thread.sleep(10000);
    // System.out.println("finished");
    //
    // System.out.println(d.getLocationConverter().fetchComplete(new
    // Location("chromosome:22")));
    //				
    // System.out.println("before fetch");
    // printMemory();

    // Location loc = new Location("chromosome:22:21m-21.5m");
    // SimpleTimer t = new SimpleTimer().start();
    // printMemory();
    // long before = printMemory().used;
    // List afs = d.getAffyFeatureAdaptor().fetch(loc);
    // long after = printMemory().used;
    // //List ts = d.getTranscriptAdaptor().fetch(loc);
    // System.out.println("mem usage: "+(after-before)+ "\ngoing to sleep....");
    // //System.out.println(afs.get(0));
    // //System.out.println(ts.get(0));
    //		 
    // System.out.println(t.getDurationInSecs());
    // Thread.sleep(100000);
    // System.out.println(afs.size() + "\t" + t.stop().getDurationInSecs());
    // printMemory();
    // System.out.println("delete afs...");
    // afs=null;
    // t.start();
    // afs = d.getAffyFeatureAdaptor().fetchUniqueProbeAndLocation(loc);
    // System.out.println(afs.size() + "\t" + t.stop().getDurationInSecs());

    // printMemory();

  }

  /**
   *  
   */
  private static SystemUtil.MemoryStatus printMemory() {
    System.gc();
    SystemUtil.MemoryStatus ms = SystemUtil.memoryStatus();
    System.out.println(ms);
    return ms;
  }
}