package localstuff;

import java.util.Iterator;
import java.util.List;

import org.ensembl.datamodel.Transcript;
import org.ensembl.driver.SequenceAdaptor;
import org.ensembl.driver.TranscriptAdaptor;
import org.ensembl.registry.DriverGroup;
import org.ensembl.registry.Registry;
import org.ensembl.util.SequenceUtil;
import org.ensembl.variation.datamodel.TranscriptVariation;
import org.ensembl.variation.datamodel.VariationFeature;
import org.ensembl.variation.driver.TranscriptVariationAdaptor;

public class VariationSeqBugReport {

  public static void main(String[] args) throws Exception {
    DriverGroup human = Registry.createDefaultRegistry().getGroup("human");
    TranscriptAdaptor ta = human.getCoreDriver().getTranscriptAdaptor();
    SequenceAdaptor sa = human.getCoreDriver().getSequenceAdaptor();
    TranscriptVariationAdaptor tva = human.getVariationDriver()
        .getTranscriptVariationAdaptor();

    // note ENST00000267363 not in ensembl version 38
    String[] stableIDs = new String[] { "ENST00000267363", "ENST00000382821", "ENST00000328604" };
    for (int i = 0; i < stableIDs.length; i++) {
      
      String stableID = stableIDs[i];
      Transcript t = ta.fetch(stableID);
      if (t==null)
        continue;

      // must get *CDNA sequence* and not transcript.getSequence()
      String transcriptSequence = sa.fetch(t.getCDNALocation()).getString();
      List variations = tva.fetch(t);
      for (Iterator iter = variations.iterator(); iter.hasNext();) {

        TranscriptVariation tvar = (TranscriptVariation) iter.next();
        
        // skip non-coding changes because they don't affect CDNA
        if (tvar.getConsequenceType().indexOf("SYNONYMOUS_CODING") == -1)
          continue;
        
        int st = tvar.getCDNAstart();
        int len =  tvar.getCDNAend() - tvar.getCDNAstart();
        
        // skip non-SNPs
        if (len>0)
          continue;
        
        // cdnaStart>=1 therefore -1 to get string position
        char tvar_char = transcriptSequence.charAt(st - 1);
        VariationFeature vf = tvar.getVariationFeature();
        String temp = vf.getSequence().getString();
        // variation_feature may be on opposite strand to transcript
        if (vf.getLocation().getStrand()!=t.getLocation().getStrand())
          temp = SequenceUtil.reverseComplement(temp);
        char vf_char = temp.charAt(0);
        
        System.out.println((vf_char==tvar_char ? "same" : "DIFF") + "\t"+ tvar_char 
            + "\t" + vf_char +"\t" + stableID + "\t" + "tvar="+tvar.getInternalID() + "\t" + tvar.getConsequenceType() 
            + "\t" + st + "\t"+ len+"\t"+tvar);

      }
    }
  }
}