package localstuff;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;

import org.ensembl.datamodel.CoordinateSystem;
import org.ensembl.datamodel.Gene;
import org.ensembl.datamodel.Location;
import org.ensembl.driver.AdaptorException;
import org.ensembl.driver.ConfigurationException;
import org.ensembl.driver.CoreDriver;
import org.ensembl.driver.CoreDriverFactory;
import org.ensembl.driver.GeneAdaptor;
import org.ensembl.driver.impl.CoreDriverImpl;

public class masterList{

  /**
  * driver to the database
  */
  private CoreDriver DBdriver;

  public masterList(){
     /* try to load driver
        if loading driver fails then try to load default driver from
        "data/default_drivers/human.conf" file
     */
     try {
         DBdriver = initializeDriver();
         System.out.println("CoreDriver loaded succesfuly");
     } catch (AdaptorException ae) {
         System.out.println("Could not load driver." +
          "Trying to load default driver..");
         try {
             DBdriver = initialiseDefaultDriver();
             System.out.println("Default driver loaded succesfuly");
         } catch (ConfigurationException ce) {
             System.out.println(ce.getMessage());
             ce.printStackTrace();
         }
     }

 }

  /**
  * fetches and displays the genes from a chromosome
  * @param driver driver to execute queries against
  * @param chromosome a string representimg the chromosome to fetch the genes
from
  * @throws AdaptorException if problem occurs during retrieval
  */
  public void fetchGenesFromChromosome(CoreDriver driver, String chromosome)
          throws AdaptorException{

     List genes = null;

     //get the gene adaptor
     GeneAdaptor geneAdaptor = driver.getGeneAdaptor();
     // fetch the gene from the location.
     try {
          genes = geneAdaptor. fetch(new Location("chromosome:11:1m-2m:-1"));
     } catch (ParseException e) {
         System.out.println(e.getMessage());
         e.printStackTrace();
         // this is another method to create a location of a chromosome.
         // this is just in case the string 
         // "chromosome:11:1m-2m:-1" suppied as argument to
         // the constructor of the Location is not ok.

         // Create an assembly location. Assembly locations represent part of a
         // genome assembly, in this case part of chromosome 11.
          Location location = new Location(new CoordinateSystem("chromosome"),
                  "11",// chromosome name
                    1, // start
                    2, // end
                   -1); // strand
          genes = geneAdaptor.fetch(location);
     }

      if (genes == null || genes.size() == 0) {
                System.out.println("No Genes found.");
      } else {
            Iterator i =genes.iterator();
            while (i.hasNext())
            {
                  Gene gene = (Gene) i.next();
                //print several information about gene.
                  System.out.println(gene.getDescription());
                  System.out.println("Gene name"+gene.getDisplayName()
                      +"InternalID = "+gene.getInternalID());
                  String[] interproIDs = gene.getInterproIDs();
                  for(int j=0;j<interproIDs.length;j++)
                      System.out.println(interproIDs[j]);
                  System.out.println("");
           }
     }

     // print the genes id to a file
      printGenesIDToFile(genes,"gene_IDs.txt");
}

  public void printGenesIDToFile(List genes, String fileName){
      if (genes == null || genes.size() == 0) {
                System.out.println("No Genes found.");
      } else {
          try {
              ObjectOutputStream oos = new ObjectOutputStream(new
FileOutputStream(fileName));
              for( Iterator i =genes.iterator();i.hasNext();){
                      Gene gene = (Gene) i.next();
                      oos.writeLong(gene.getInternalID());
               }
               oos.close();
          }catch (IOException e) {
              e.printStackTrace();
              System.out.println(e.getMessage());
          }
      }
  }

  public CoreDriver getDBdriver() {
        return DBdriver;
    }

  //driver for the latest human release.
  private CoreDriver initialiseDefaultDriver() throws ConfigurationException{
    
      try {
        return CoreDriverFactory.createCoreDriver("data/default_drivers/human.conf");
      } catch (AdaptorException e) {
        throw new ConfigurationException("",e);
      }
      
    }

 //host=ensembldb.ensembl.org
 //user=anonymous
 //database=homo_sapiens_core_22_34d
  private CoreDriver initializeDriver() throws AdaptorException {
        try {
            Class.forName("com.mysql.jdbc.CoreDriver");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        CoreDriver driver = new CoreDriverImpl("ensembldb.ensembl.org", //host
                                    "homo_sapiens_core_22_34d",  //database
                                    "anonymous");                //user
        return driver;
    }

  public static void main(String[] args) {
        masterList m=new masterList();

        try {
          m.fetchGenesFromChromosome(m.getDBdriver(),"");
        } catch (AdaptorException e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }

    }
} 
